#! /usr/bin/env python

"""
The preparation module is provide to isolate methods needed to run Phase of Operations
 preparations phase (1.2.2.1), including all sub phases:
   - Data Generation (1.2.2.1.1)
   - Data Warehouse Creation (1.2.2.1.2)
   - Data Integration Preparation (1.2.2.1.3)

http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=14
"""

import shlex
import subprocess
import sys
import os
import zipfile
import inspect
import requests
import json
from os.path import exists as path_exists
from subprocess import check_output

from pyspark.sql.session import SparkSession

from .staging.staging import Staging
from .warehouse.warehouse import Warehouse
from .transformation.historicalClass import HistoricalClass
from .transformation.incrementalClass import IncrementalClass
from .validation.validation import Validation


def sh(args):
    return str(check_output(args)).split('\\n')


class PreparationPhase:
    spark: SparkSession
    sf: int
    stg_name: str
    wh_name: str
    stg_dir_local: str
    pdgf_gen_dir_parent: str
    staging: Staging
    warehouse: Warehouse
    validation: Validation

    def run_data_generation(self):
        lib_tool_zip_path = os.path.abspath(os.path.join(inspect.getfile(PreparationPhase),
                                                         '../preparation/tpc-di-tool.zip'))
        pdgf_gen_tool_zip_path = self.pdgf_gen_dir_parent + '/tpc-di-tool.zip'
        pdgf_gen_tool_path = self.pdgf_gen_dir_parent + '/Tools'
        pdgf_jar_path = self.pdgf_gen_dir_parent + '/Tools/DIGen.jar'
        # Make sure stg_dir exists, create if doesn't
        if path_exists(self.stg_dir_local):
            if path_exists(self.stg_dir_local + '/Batch1'):
                print('Data has already been generated.')
                return None
            else:
                if ~path_exists('/'.join(self.stg_dir_local.split('/')[:-1])):
                    # Action if stg db does not exist, create database which creates parent folder
                    spark = SparkSession.builder.getOrCreate()
                    stg_name = self.stg_dir_local.split('/')[-2].split('.')[0]
                    print('Create Database: ' + stg_name)
                    spark.sql(f"CREATE DATABASE IF NOT EXISTS {stg_name}")
                sh(['mkdir', '-p', self.stg_dir_local])
                print('Create staging directory: ' + self.stg_dir_local)
        # Make sure pdgf_gen_dir_parent exists, create if it doesn't
        if ~path_exists(self.pdgf_gen_dir_parent):
            sh(['mkdir', '-p', self.pdgf_gen_dir_parent])
            print('Create PDGF generation directory: ' + self.pdgf_gen_dir_parent)
        # Make sure tpc-di-tool.zip exists, action if isn't
        # TODO: Check licensing limitations
        if ~path_exists(pdgf_gen_tool_zip_path):
            sh(['cp', lib_tool_zip_path, self.pdgf_gen_dir_parent])
            print('Copy tpc-di-tool.zip: ' + pdgf_gen_tool_zip_path)
        # Make sure Tools path exists, unzip if doesn't
        if ~path_exists(pdgf_gen_tool_path):
            with zipfile.ZipFile(pdgf_gen_tool_zip_path, 'r') as zip_ref:
                zip_ref.extractall(self.pdgf_gen_dir_parent)
        # Run generator
        cmd = f"java -jar {pdgf_jar_path} -sf {self.sf} -o {self.stg_dir_local}"
        print(f"Generating data and outputting to {self.stg_dir_local}")
        args = shlex.split(cmd)
        p3 = subprocess.Popen(args,
                              cwd=pdgf_gen_tool_path,
                              # shell=True,
                              universal_newlines=True,
                              stdin=subprocess.PIPE,
                              stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
        p3.stdin.write("\n")
        p3.stdin.flush()
        p3.stdin.write("YES\n")
        p3.stdin.flush()
        while True:
            output = p3.stdout.readline()
            if p3.poll() is not None and output == '':
                break
            if output:
                print(output.strip())
        p3.wait()
        self._set_historical_incremental_validation()

    def run_data_gen_cleanup(self):
        sh(['rm', '-rf', self.stg_dir_local])
        sh(['rm', '-rf', self.pdgf_gen_dir_parent])

    def run_warehouse_creation(self):
        self.warehouse.create()
        self.warehouse.create_tables()

    @staticmethod
    def run_integration_preparation(domain, token, cluster_id):
        response = requests.post('%s/api/2.0/libraries/install' % domain,
                                 headers={'Authorization': 'Bearer %s' % token},
                                 json={"cluster_id": cluster_id,
                                       "libraries": [{"maven": {"coordinates": "com.databricks:spark-xml_2.12:0.14.0"}},
                                                     {"pypi": {"package": "tpcdi"}},
                                                     {"pypi": {"package": "graphviz"}}]})
        init_script_source_path = os.path.abspath(os.path.join(inspect.getfile(PreparationPhase),
                                                               '../preparation/preparation_init.sh'))
        init_script_target_parent = '/dbfs/Shared/init_scripts'
        init_script_target_path = '/dbfs/Shared/init_scripts/preparation_init.sh'
        init_script_target_dbfs = 'dbfs:/Shared/init_scripts/preparation_init.sh'
        # Make sure init_script_target_parent exists, create if doesn't
        if ~path_exists(init_script_target_parent):
            sh(['mkdir', '-p', init_script_target_parent])
        # Make sure init_script_target_path exists, create if doesn't
        if ~path_exists(init_script_target_path):
            sh(['cp', '-p', init_script_source_path, init_script_target_path])
        response = requests.get('%s/api/2.0/clusters/get' % domain,
                                headers={'Authorization': 'Bearer %s' % token},
                                json={"cluster_id": cluster_id})
        cluster_conf_json = json.loads(response.text)
        cluster_edit_json = {}
        cluster_edit_json['cluster_id'] = cluster_conf_json['cluster_id']
        cluster_edit_json['cluster_name'] = cluster_conf_json['cluster_name']
        cluster_edit_json['spark_conf'] = cluster_conf_json['spark_conf']
        cluster_edit_json['custom_tags'] = cluster_conf_json['custom_tags']
        cluster_edit_json['num_workers'] = cluster_conf_json['num_workers']
        cluster_edit_json['spark_version'] = cluster_conf_json['spark_version']
        cluster_edit_json['node_type_id'] = cluster_conf_json['node_type_id']
        cluster_edit_json['init_scripts'] = [{"dbfs": {"destination": init_script_target_dbfs}}]
        response = requests.post('%s/api/2.0/clusters/edit' % domain,
                                 headers={'Authorization': 'Bearer %s' % token},
                                 json=cluster_edit_json)

    def run_preparation_phase(self, domain, token, cluster_id):
        self.run_data_generation()
        self.run_warehouse_creation()
        self.run_integration_preparation(domain, token, cluster_id)

    def _set_historical_incremental_validation(self):
        # Load historical / incremental only if staging data exists
        self.historical = HistoricalClass(self.staging, self.warehouse)
        self.incremental_1 = IncrementalClass(self.staging, self.warehouse, 2)
        self.incremental_2 = IncrementalClass(self.staging, self.warehouse, 3)
        self.validation = Validation(self.warehouse)


def preparation_main(sf, domain, token, cluster_id):
    pp = PreparationPhase()
    pp.staging = Staging(f'tpcdi_stg_{sf}',
                         f'dbfs:/user/hive/warehouse/tpcdi_stg_{sf}.db/pdgf_output')
    pp.warehouse = Warehouse(f'tpcdi_wh_{sf}')
    pp.run_preparation_phase(domain, token, cluster_id)


if __name__ == "__main__":
    scale_factor = int(sys.argv[1])
    domain = int(sys.argv[2])
    token = int(sys.argv[3])
    cluster_id = int(sys.argv[4])
    preparation_main(scale_factor,
                     domain,
                     token,
                     cluster_id)
