"""
Functions for Staging Account.txt Files

definition is taken exactly from table 2.2.3

TPC-DI Requirement: 2.2.2.1
TPC-DI Ref: http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=21

https://kb.databricks.com/scala/pattern-match-files-in-path.html
"""

from ..stgData import StgData
from ..fileFormatDef import *
from pyspark.sql.types import StructType, StructField
from pyspark.sql.functions import lit, regexp_replace, split, col, when, array, struct, length, to_timestamp


CMP = [("CompanyName",   60,  True, "Not empty Name of the company"),
       ("CIK",           10,  True, "Not empty Company identification code from SEC"),
       ("Status",        4,   True, "Not empty ‘ACTV’ for Active company, ‘INAC’ for inactive"),
       ("IndustryID",    2,   True, "Not empty Code for industry segment"),
       ("SPrating",      4,   True, "Not empty S&P rating"),
       ("FoundingDate",  8,   True, "A date as YYYYMMDD"),
       ("AddrLine1",     80,  True, "Not empty Mailing address"),
       ("AddrLine2",     80,  True, "Mailing address"),
       ("PostalCode",    12,  True, "Not empty Mailing address"),
       ("City",          25,  True, "Not empty Mailing address"),
       ("StateProvince", 20,  True, "Not empty Mailing address"),
       ("Country",       24,  True, "Mailing address"),
       ("CEOname",       46,  True, "Not empty Name of company CEO"),
       ("Description",   150, True, "Not empty Description of the company")]

SEC = [("Symbol",          15, True, "Security symbol"),
       ("IssueType",       6,  True, "Issue type"),
       ("Status",          4,  True, "‘ACTV’ for Active security, ‘INAC’ for inactive"),
       ("Name",            70, True, "Security name"),
       ("ExID",            6,  True, "ID of the exchange the security is traded on"),
       ("ShOut",           13, True, "Number of shares outstanding"),
       ("FirstTradeDate",  8,  True, "Date of first trade as YYYYMMDD"),
       ("FirstTradeExchg", 8,  True, "Date of first trade on exchange as YYYYMMDD"),
       ("Dividend",        12, True, "Dividend as VALUE_T"),
       ("CIK",             10, True,  "Company CIK number (if last len 10)"),
       ("CoName",          60, True,  "Company name (if last len 60)")]

FIN = [("Year",         4,  True, "Year of the quarter end."),
       ("Quarter",      1,  True, "Quarter number: valid values are ‘1’, ‘2’, ‘3’, ‘4’"),
       ("QtrStartDate", 8,  True, "Start date of quarter, as YYYYMMDD"),
       ("PostingDate",  8,  True, "Posting date of quarterly report as YYYYMMDD"),
       ("Revenue",      17, True, "Reported revenue for the quarter"),
       ("Earnings",     17, True, "Net earnings reported for the quarter"),
       ("EPS",          12, True, "Basic earnings per share for the quarter"),
       ("DilutedEPS",   12, True, "Diluted earnings per share for the quarter"),
       ("Margin",       12, True, "Profit divided by revenues for the quarter"),
       ("Inventory",    17, True, "Value of inventory on hand at end of quarter"),
       ("Assets",       17, True, "Value of total assets at the end of quarter"),
       ("Liabilities",  17, True, "Value of total liabilities at the end of quarter"),
       ("ShOut",        13, True, "Average number of shares outstanding"),
       ("DilutedShOut", 13, True, "Average number of shares outstanding (diluted)"),
       ("CIK",          10, True,  "Company CIK number (if last len 10)"),
       ("CoName",       60, True,  "Company name (if last len 60)")]


class FINWIRE(StgData):

    file_name = "FINWIRE[0-9][0-9][0-9][0-9]Q[1-4]"
    batches = [1, ]
    definition = [("PTS",      CHAR(15), True, "Posting date & time as YYYYMMDD-HHMMSS"),
                  ("RecType",  CHAR(3),  True, "Record Type; CMP, SEC, FIN"),
                  ("CMP",      StructType([StructField(c[0], CHAR(c[1]), c[2]) for c in CMP]), True, "CMP Detail"),
                  ("SEC",      StructType([StructField(c[0], CHAR(c[1]), c[2]) for c in SEC]), True, "SEC Detail"),
                  ("FIN",      StructType([StructField(c[0], CHAR(c[1]), c[2]) for c in FIN]), True, "FIN Detail")]

    def get_df(self, batch_id):
        batch_id = 1 if batch_id != 1 else batch_id

        def splitter(desc, pickLast=False):
            widthCols = [f[1] for f in desc[:-1]]
            cnt = len(widthCols)-pickLast
            regex_pat = "".join(['(.{'+str(n)+'})' for n in widthCols[:cnt]])+'(.*)$'
            regex_val = "\001".join(['$'+str(n+1) for n in range(cnt+1)])
            return split(regexp_replace(col('value').getItem(2), regex_pat, regex_val), '\001')
                  
        def structer(nameCol, desc, pickLast=False):
            if pickLast:
                cols = [col(nameCol).getItem(i).alias(c[0]) for i,c in enumerate(desc[:-2])]  
                cols += [when(length(col(nameCol).getItem(len(desc)-2))==lit(desc[-2][1]), col(nameCol).getItem(len(desc)-2)).alias(desc[-2][0])]
                cols += [when(length(col(nameCol).getItem(len(desc)-2))!=lit(desc[-2][1]), col(nameCol).getItem(len(desc)-2)).alias(desc[-1][0])]
            else:
                cols = [col(nameCol).getItem(i).alias(c[0]) for i, c in enumerate(desc)]
            return when(col(nameCol).isNotNull(), struct(cols))

        return self.spark.read.format("text").load(f'{self.stg_dir}/Batch{batch_id}/{self.file_name}') \
                   .withColumn("value", split(regexp_replace(col('value'),'(.{15})(.{3})(.*)','$1\001$2\001$3'), '\001')) \
                   .withColumn("PTS", to_timestamp(col('value').getItem(0), "yyyyMMdd-HHmmss")) \
                   .withColumn("RecType", col('value').getItem(1)) \
                   .withColumn("CMP",  when(col("RecType")==lit("CMP"), splitter(CMP, pickLast=False))) \
                   .withColumn("SEC",  when(col("RecType")==lit("SEC"), splitter(SEC, pickLast=True))) \
                   .withColumn("FIN",  when(col("RecType")==lit("FIN"), splitter(FIN, pickLast=True))) \
                   .drop(col('value')) \
                   .withColumn("CMP", structer("CMP", CMP, pickLast=False)) \
                   .withColumn("SEC", structer("SEC", SEC, pickLast=True)) \
                   .withColumn("FIN", structer("FIN", FIN, pickLast=True))

