"""
Provide Graphic to list tables in Warehouse

All of the tables in the Data Warehouse are listed 1.4 Destination Data Model. However, presentation and 
ease of inspection would be tedious for users either not familiar with Databricks or the TPC-DI benchmark.
Thus, we'll provide a graphic that is visually similar to Figure 1.4-1 and will hyperlink to the data warehouse 
tables that would be created with the given sf.

NOTE: The code segment, gvDependency.py, is not used in any phases of operation tasks. It is provided only for explainability
      in interactive notebook sessions.

TPC-DI Requirement: 1.4
TPC-DI Ref: http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=17
"""


def get_stg_html(stg_name):
    import re
    from graphviz import Graph

    dot = Graph('tpcdi')
    dot.graph_attr['rankdir'] = 'LR'
    dot.attr('node', shape='rectangle')

    def tbl_link(tbl_name, ttip=''):
        return {'tooltip': ttip, 'href': f'#table/hive_metastore/{stg_name}/{tbl_name}', 'target': "_blank"}

    with dot.subgraph(name='cluster_staging') as cs:
        cs.node('Date', 'Date', fillcolor='lightgray', style='filled',
                **tbl_link('date', ttip='2.2.2.7'))
        cs.node('Time', 'Time', fillcolor='lightgray', style='filled',
                **tbl_link('time', ttip='2.2.2.15'))
        cs.node('RawIndustry', 'Industry', fillcolor='lightgray', style='filled',
                **tbl_link('rawindustry', ttip='2.2.2.11'))
        cs.node('RawStatusType', 'StatusType', fillcolor='lightgray', style='filled',
                **tbl_link('rawstatustype', ttip='2.2.2.13'))
        cs.node('RawTaxRate', 'TaxRate', fillcolor='lightgray', style='filled',
                **tbl_link('rawtaxrate', ttip='2.2.2.14'))
        cs.node('RawTradeType', 'TradeType', fillcolor='lightgray', style='filled',
                **tbl_link('rawtradetype', ttip='2.2.2.18'))
        cs.node('Trade', 'Trade', fillcolor='lightgray', style='filled',
                **tbl_link('trade', ttip='2.2.2.17'))
        cs.node('TradeHistory', 'TradeHistory', fillcolor='lightgray', style='filled',
                **tbl_link('tradehistory', ttip='2.2.2.16'))
        cs.node('CashTransaction', 'CashTransaction', fillcolor='lightgray', style='filled',
                **tbl_link('cashtransaction', ttip='2.2.2.3'))
        cs.node('HoldingHistory', 'HoldingHistory', fillcolor='lightgray', style='filled',
                **tbl_link('holdinghistory', ttip='2.2.2.9'))
        cs.node('DailyMarket', 'DailyMarket', fillcolor='lightgray', style='filled',
                **tbl_link('dailymarket', ttip='2.2.2.6'))
        cs.node('WatchHistory', 'WatchHistory', fillcolor='lightgray', style='filled',
                **tbl_link('watchhistory', ttip='2.2.2.19'))
        cs.node('HR', 'HR', fillcolor='lightgray', style='filled',
                **tbl_link('hr', ttip='2.2.2.10'))
        cs.node('RawProspect', 'Prospect', fillcolor='lightgray', style='filled',
                **tbl_link('rawprospect', ttip='2.2.2.12'))
        cs.node('FINWIRE', 'FINWIRE', fillcolor='lightgray', style='filled',
                **tbl_link('finwire', ttip='2.2.2.8'))
        cs.node('CustomerMgmt', 'CustomerMgmt', fillcolor='lightgray', style='filled',
                **tbl_link('customermgmt', ttip='2.2.2.5'))
        cs.node('Account', 'Account', fillcolor='lightgray', style='filled',
                **tbl_link('tradehistory', ttip='2.2.2.16'))
        cs.node('Customer', 'Customer', fillcolor='lightgray', style='filled',
                **tbl_link('customer', ttip='2.2.2.4'))
        cs.body.append('label="Staging Database Tables"')

    dot.edge('Date', 'Trade', style="invis")
    dot.edge('Date', 'TradeHistory', style="invis")
    dot.edge('Time', 'TradeHistory', style="invis")
    dot.edge('Time', 'CashTransaction', style="invis")
    dot.edge('RawIndustry', 'CashTransaction', style="invis")
    dot.edge('RawIndustry', 'HoldingHistory', style="invis")
    dot.edge('RawStatusType', 'HoldingHistory', style="invis")
    dot.edge('RawStatusType', 'DailyMarket', style="invis")
    dot.edge('RawTaxRate', 'DailyMarket', style="invis")
    dot.edge('RawTaxRate', 'WatchHistory', style="invis")
    dot.edge('RawTradeType', 'WatchHistory', style="invis")
    dot.edge('RawTradeType', 'HR', style="invis")

    dot.edge('Trade', 'RawProspect', style="invis")
    dot.edge('TradeHistory', 'RawProspect', style="invis")
    dot.edge('CashTransaction', 'RawProspect', style="invis")
    dot.edge('CashTransaction', 'FINWIRE', style="invis")
    dot.edge('HoldingHistory', 'FINWIRE', style="invis")
    dot.edge('DailyMarket', 'FINWIRE', style="invis")
    dot.edge('DailyMarket', 'CustomerMgmt', style="invis")
    dot.edge('WatchHistory', 'CustomerMgmt', style="invis")
    dot.edge('HR', 'CustomerMgmt', style="invis")

    dot.edge('RawProspect', 'Account', style="invis")
    dot.edge('FINWIRE', 'Account', style="invis")
    dot.edge('FINWIRE', 'Customer', style="invis")
    dot.edge('CustomerMgmt', 'Customer', style="invis")

    html = dot._repr_image_svg_xml()

    html = re.sub(r'<svg width=\"\d*pt\" height=\"\d*pt\"',
                  '<div style="text-align:center;"><svg height="400pt" aligned=center', html)
    html = re.sub(r'</svg>', '</div</svg>', html)

    return html
