"""
Parent Class For stgData classes

There are a couple conventions here that should be observed:
 - The name of the table will be the name of the class by default
 - stg_name is derived from stg_dir

NOTABLE CONVENTIONS - There are some impolicit conventions that were in the TPC-DI documentation 
that are codified in StgData:
 - The file extension 'txt' is delimited by '|', 'csv' by ','
 - omittedHistoricCDC: There is a convention for loading history of CDC data where the fields
   '' and '' are not provided in historic data. These data sources are identifiable by the inclusion
   of this field in their definition.
"""

from delta import DeltaTable
from pyspark.sql.functions import lit
from pyspark.sql.types import StructType, StructField, StringType, DecimalType

class StgData:

    file_name = 'name.ext'
    batches = [1, 2, 3]
    definition = [("Column Name", "DataType", "Nullable", "Comment")]

    def __init__(self, spark, stg_name, stg_dir):
        self.spark = spark
        self.stg_name = stg_name
        self.tbl_name = self.__class__.__name__
        self.tbl = f'{self.stg_name}.{self.tbl_name}'
        self.stg_dir = stg_dir

    def get_schema(self):
        return StructType([StructField(*c[:-1]) for c in self.definition])

    def get_df(self, batch_id):
        if batch_id not in self.batches:
            return None
        delimiter = "," if self.file_name.split('.')[-1] == "csv" else "|"
        if (batch_id==1) & (self.definition[0][0] == 'CDC_FLAG'):
            schema_b1 = StructType([StructField(*c[:-1]) for c in self.definition if c[0] not in ('CDC_FLAG', 'CDC_DSN')])
            return self.spark.read.format("csv") \
                        .schema(schema_b1) \
                        .option("delimiter", delimiter) \
                        .load(f'{self.stg_dir}/Batch1/{self.file_name}') \
                        .withColumn('CDC_FLAG', lit(None).cast(StringType())) \
                        .withColumn('CDC_DSN', lit(None).cast(DecimalType(11,0))) \
                        .select([c[0] for c in self.definition])
        return self.spark.read.format("csv") \
                        .schema(self.get_schema()) \
                        .option("delimiter", delimiter) \
                        .load(f'{self.stg_dir}/Batch{batch_id}/{self.file_name}')

    def create_table(self):
        dt = DeltaTable.createIfNotExists(self.spark).tableName(self.tbl)
        for c in self.definition:
            dt.addColumn(*c[:-1], comment=c[-1])
        if self.tbl_name != "Audit":
            dt.addColumn("BatchID", "DECIMAL(5)", comment='BatchID the data is associated with')
        dt.partitionedBy("BatchID") \
          .execute()

    def drop_table(self):
        self.spark.sql(f'DROP TABLE IF EXISTS {self.tbl};')

    def ingest(self):
        for batch_id in self.batches:
            self.get_df(batch_id) \
                .withColumn("BatchId", lit(batch_id)) \
                .write.mode("overwrite") \
                .insertInto(self.tbl)

    def get_table(self):
        # TODO: Check for existence
        return self.spark.table(self.tbl)

    def create_view(self):
        # TODO: Check for existence
        return self.get_table().createOrReplaceTempView(self.tbl_name)
