"""
Alerts don't exist for all data sources. However, when they do exist they can consistently
be defined from the HistoricLoad/IncrementalUpdate mdf. Thus, for ease of maintenance we
will consolidate all alerts in alert.py




DimTrade - 4.5.8.3, 4.5.8.4, 4.6.7.3, 4.6.7.4
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=65
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=74

FactMarketHistory - 4.5.11.3, 4.6.10.2
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=65
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=77

Prospect - 4.5.15.2 (Status Message) + 4.6.14.2
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=71
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=779

Likely best to check out:
https://docs.databricks.com/delta/delta-utility.html#history-schema

"""

from pyspark.sql import DataFrame
from pyspark.sql.functions import lit, col, concat, struct, when, array, array_except, explode
from datetime import datetime
from datetime import timedelta


def dimcompany_alert(dimcompany_mdf: DataFrame,
                     batch_id: int,
                     current_timestamp: str) -> DataFrame:
    """
    DimCompany Alert - Invalid SPRating 4.5.3.3
    http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=61
    NOTE: mdf already has a computed SPRating and null values are invalid
    """
    return dimcompany_mdf \
        .filter(col('SPrating').isNull()) \
        .select(lit(current_timestamp).alias('MessageDateAndTime'),
                lit(batch_id).alias('BatchID'),
                lit('DimCompany').alias('MessageSource'),
                lit('Invalid SPRating').alias('MessageText'),
                lit('Alert').alias('MessageType'),
                concat(lit('CO_ID = '), col('CompanyID'), lit(', CO_SP_RATE = '),
                       col('SPRating')).alias('MessageData'))


def dimcustomer_alert(dimcustomer_mdf: DataFrame,
                      batch_id: int,
                      current_timestamp: str,
                      batch_date: str) -> DataFrame:
    """
    DimCustomer - 4.5.4.4, 4.5.4.5, 4.6.4.4, 4.6.4.5
    http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=64
    http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=73
    """
    dob_not_between = ~col("DOB").between(datetime.strptime(batch_date, '%Y-%m-%d') - timedelta(days=365.25 * 100 + 1),
                                          datetime.strptime(batch_date, '%Y-%m-%d') + timedelta(days=1))
    return dimcustomer_mdf.select(
        explode(array_except(array(when(~col("Tier").isin([1, 2, 3]),
                                        struct(lit(current_timestamp).alias('MessageDateAndTime'),
                                               lit(batch_id).alias('BatchID'),
                                               lit('DimCompany').alias('MessageSource'),
                                               lit('Invalid customer tier').alias('MessageText'),
                                               lit('Alert').alias('MessageType'),
                                               concat(lit('CO_ID = '), col('CustomerID'),
                                                      lit(', C_TIER = '), col("Tier")).alias('MessageData'))),
                                   when(dob_not_between,
                                        struct(lit(current_timestamp).alias('MessageDateAndTime'),
                                               lit(batch_id).alias('BatchID'),
                                               lit('DimCompany').alias('MessageSource'),
                                               lit('DOB out of range').alias('MessageText'),
                                               lit('Alert').alias('MessageType'),
                                               concat(lit('CO_ID = '), col('CustomerID'),
                                                      lit(', C_DOB = '), col('DOB')).alias('MessageData')))),
                             array(lit(None)))).alias('msgDetail')).select('msgDetail.*').dropDuplicates()
