"""
Historic WorkFlow Part, DimCompany
**HISTORY TRACKING TABLE**


Transform Reference: 4.5.3
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=60
"""

from ..hl_transform import Hl_transform
from pyspark.sql.functions import col, lit, row_number, regexp_extract, to_date, when, date_add, lead, concat
from pyspark.sql.window import Window as W
from pyspark.sql import DataFrame


class Hl_DimCompany(Hl_transform):
    source_names = {'staging': [],
                    'warehouse': ['AuxFINWIRE_CMP',
                                  'StatusType',
                                  'Industry']}

    def _get_load_df(self) -> DataFrame:
        w_pts = W.partitionBy(col('CIK')).orderBy(col('dte'))
        w_row = W.partitionBy(lit(1)).orderBy(lit(1))

        return self.source['AuxFINWIRE_CMP']() \
            .join(self.source['StatusType'](), col('Status') == col("ST_ID"), "left") \
            .join(self.source['Industry'](), col('IndustryID') == col("IN_ID"), "left") \
            .withColumn('dte', to_date(col('PTS'))) \
            .withColumn('lead_dte_minus_1', date_add(lead(col('dte')).over(w_pts), -1)) \
            .withColumn('SPrating_formatted',
                        regexp_extract(col('SPRating'), r"^([A-C]{1,3}(?:\+|-)?|D)\s*$", 1)) \
            .withColumn('SPrating_lowgrade', col('SPRating').rlike('^A|(BBB)].*')) \
            .select(row_number().over(w_row).alias("SK_CompanyID"),
                    col('CIK').alias('CompanyID'),
                    col('ST_NAME').alias('Status'),
                    col('CompanyName').alias('Name'),
                    col('IN_NAME').alias('Industry'),
                    when(col('SPrating_formatted') != '', col('SPrating_formatted')).alias('SPrating'),
                    when(col('SPrating_lowgrade').isNull(), lit(True)).otherwise(lit(False)).alias(
                        'isLowGrade'),
                    col('CEOname').alias('CEO'),
                    when(col('AddrLine1') != 80 * ' ', col('AddrLine1')).alias('AddressLine1'),
                    when(col('AddrLine2') != 80 * ' ', col('AddrLine2')).alias('AddressLine2'),
                    when(col('PostalCode') != 12 * ' ', col('PostalCode')).alias('PostalCode'),
                    when(col('City') != 25 * ' ', col('City')).alias('City'),
                    when(col('StateProvince') != 20 * ' ', col('StateProvince')).alias('StateProv'),
                    when(col('Country') != 24 * ' ', col('Country')).alias('Country'),
                    col('Description'),
                    col('FoundingDate'),
                    when(col('lead_dte_minus_1').isNull(), lit(True)).otherwise(lit(False)).alias("IsCurrent"),
                    lit(1).alias("BatchID"),
                    col('dte').alias('EffectiveDate'),
                    when(col('lead_dte_minus_1').isNull(),
                         "9999-12-31").otherwise(col('lead_dte_minus_1')).alias('EndDate'),
                    when(col('SPrating_formatted') == '', col('SPrating')).alias('SPrating_invalid'))

    def _get_alert_df(self) -> DataFrame:
        """
        DimCompany Alert - Invalid SPRating 4.5.3.3
        http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=61
        NOTE: mdf already has a computed SPRating and null values are invalid
        """
        return self.load_df \
            .filter(col('SPrating_invalid').isNotNull()) \
            .select(lit(self.CurrentTimestamp).alias('MessageDateAndTime'),
                    lit(self.BatchID).alias('BatchID'),
                    lit('DimCompany').alias('MessageSource'),
                    lit('Invalid SPRating').alias('MessageText'),
                    lit('Alert').alias('MessageType'),
                    concat(lit('CO_ID = '), col('CompanyID'), lit(', CO_SP_RATE = '),
                           col('SPRating_invalid')).alias('MessageData'))

    def run_insert(self):
        self.set_load_df()
        if self.load_df is not None:
            self.load_df.drop('SPRating_invalid').write.insertInto(self.target_name)
