"""
Historic WorkFlow Part, DimCustomer
**HISTORY TRACKING TABLE**


Transform Reference: 4.5.4
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=61
"""

from ..hl_transform import Hl_transform
from pyspark.sql.functions import col, lit, row_number, when, struct, upper, date_add, lead, last, to_date, \
                                  explode, array_except, array, concat
from pyspark.sql.window import Window as W
from datetime import datetime, timedelta
from pyspark.sql import DataFrame


class Hl_DimCustomer(Hl_transform):

    source_names = {'staging': [],
                    'warehouse': ['AuxCustomerMgmt',
                                  'AuxProspect',
                                  'TaxRate']}

    def _get_load_df(self) -> DataFrame:
        w_sk_cust = W.partitionBy(lit(1)).orderBy(col('ActionTS'))
        w_cust = W.partitionBy(col('C_ID')).orderBy(col('ActionTS'))
        w_cust_prev = W.partitionBy(col('C_ID')) \
            .orderBy(col('ActionTS')) \
            .rowsBetween(W.unboundedPreceding, W.currentRow)

        def cust_prev(col_value, col_alias):
            return last(col_value, True).over(w_cust_prev).alias(col_alias)

        tax_rate = self.source['TaxRate']()

        return self.source['AuxCustomerMgmt']() \
            .filter(col('ActionType').isin(["NEW", "UPDCUST", "INACT"])) \
            .withColumn('next_ActionTS_minus_1', date_add(lead(col('ActionTS')).over(w_cust), -1)) \
            .select(struct(row_number().over(w_sk_cust).alias("SK_CustomerID"),
                           col('C_ID').alias("CustomerID"),
                           cust_prev(col('C_TAX_ID'), 'TaxID'),
                           # TODO: Confirm case requirements for ('Active','ACTIVE'), ('Inactive','INACTIVE')
                           cust_prev(when(col('ActionType').isin(['NEW', 'UPDCUST']), lit('Active'))
                                     .otherwise(when(col('ActionType') == lit('INACT'), lit('Inactive'))
                                                .otherwise(lit(None))), 'Status'),
                           cust_prev(col('C_L_NAME'), 'LastName'),
                           cust_prev(col('C_F_NAME'), 'FirstName'),
                           cust_prev(col('C_M_NAME'), 'MiddleInitial'),
                           cust_prev(when(col('C_GNDR').isin(['M', 'm', 'F', 'f']),
                                          upper(col('C_GNDR'))).otherwise(lit('U')), 'Gender'),
                           cust_prev(col('C_TIER'), 'Tier'),
                           cust_prev(col('C_DOB'), 'DOB'),
                           cust_prev(col('C_ADLINE1'), 'AddressLine1'),
                           cust_prev(col('C_ADLINE2'), 'AddressLine2'),
                           cust_prev(col('C_ZIPCODE'), 'PostalCode'),
                           cust_prev(col('C_CITY'), 'City'),
                           cust_prev(col('C_STATE_PROV'), 'State_Prov'),
                           cust_prev(col('C_CTRY'), 'Country'),
                           cust_prev(col('C_PHONE_1'), 'Phone1'),
                           cust_prev(col('C_PHONE_2'), 'Phone2'),
                           cust_prev(col('C_PHONE_3'), 'Phone3'),
                           cust_prev(col('C_PRIM_EMAIL'), 'Email1'),
                           cust_prev(col('C_ALT_EMAIL'), 'Email2')).alias('CustomerDetail'),
                    cust_prev(col('C_NAT_TX_ID'), 'C_NAT_TX_ID'),
                    cust_prev(col('C_LCL_TX_ID'), 'C_LCL_TX_ID'),
                    when(col('next_ActionTS_minus_1').isNull(),
                         struct(lit(True).alias('isCurrent'),
                                lit(1).alias('BatchID'),
                                to_date(col('ActionTS')).alias('EffectiveDate'),
                                lit('9999-12-31').alias('EndDate'))).otherwise(
                        struct(lit(False).alias('isCurrent'),
                               lit(1).alias('BatchID'),
                               to_date(col('ActionTS')).alias('EffectiveDate'),
                               col('next_ActionTS_minus_1').alias('EndDate'))).alias('HistTrackDetail')) \
            .join(self.source['AuxProspect'](),
                  [col('HistTrackDetail.isCurrent'),
                   upper(col('CustomerDetail.AddressLine1')) == upper(col('AddressLine1')),
                   upper(col('CustomerDetail.LastName')) == upper(col('LastName')),
                   upper(col('CustomerDetail.PostalCode')) == upper(col('PostalCode')),
                   upper(col('CustomerDetail.FirstName')) == upper(col('FirstName')),
                   upper(col('CustomerDetail.AddressLine2')) == upper(col('AddressLine2'))], 'left') \
            .join(tax_rate.withColumnRenamed('TX_ID', 'NAT_TX_ID')
                  .withColumnRenamed('TX_NAME', 'NationalTaxRateDesc')
                  .withColumnRenamed('TX_RATE', 'NationalTaxRate'),
                  col('C_NAT_TX_ID') == col('NAT_TX_ID'), 'left') \
            .join(tax_rate.withColumnRenamed('TX_ID', 'LCL_TX_ID')
                  .withColumnRenamed('TX_NAME', 'LocalTaxRateDesc')
                  .withColumnRenamed('TX_RATE', 'LocalTaxRate'),
                  col('C_LCL_TX_ID') == col('LCL_TX_ID'), 'left') \
            .select(col("CustomerDetail.*"),
                    col("NationalTaxRateDesc"),
                    col("NationalTaxRate"),
                    col("LocalTaxRateDesc"),
                    col("LocalTaxRate"),
                    col("AgencyID"),
                    col("CreditRating"),
                    col("NetWorth"),
                    col("MarketingNameplate"),
                    col("HistTrackDetail.*"))

    def _get_alert_df(self) -> DataFrame:
        """
        DimCustomer - 4.5.4.4, 4.5.4.5, 4.6.4.4, 4.6.4.5
        http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=64
        http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=73
        """
        dob_not_between = ~col("DOB").between(
            datetime.strptime(self.BatchDate, '%Y-%m-%d') - timedelta(days=365.25 * 100 + 1),
            datetime.strptime(self.BatchDate, '%Y-%m-%d') + timedelta(days=1))
        return self.load_df.select(
                explode(array_except(array(when(~col("Tier").isin([1, 2, 3]),
                                                struct(lit(self.CurrentTimestamp).alias('MessageDateAndTime'),
                                                       lit(self.BatchID).alias('BatchID'),
                                                       lit('DimCompany').alias('MessageSource'),
                                                       lit('Invalid customer tier').alias('MessageText'),
                                                       lit('Alert').alias('MessageType'),
                                                       concat(lit('CO_ID = '), col('CustomerID'),
                                                              lit(', C_TIER = '), col("Tier")).alias('MessageData'))),
                                           when(dob_not_between,
                                                struct(lit(self.CurrentTimestamp).alias('MessageDateAndTime'),
                                                       lit(self.BatchID).alias('BatchID'),
                                                       lit('DimCompany').alias('MessageSource'),
                                                       lit('DOB out of range').alias('MessageText'),
                                                       lit('Alert').alias('MessageType'),
                                                       concat(lit('CO_ID = '), col('CustomerID'),
                                                              lit(', C_DOB = '), col('DOB')).alias('MessageData')))),
                                     array(lit(None)))).alias('msgDetail')).select('msgDetail.*')
