"""
Historic WorkFlow Part, DimTrade
**HISTORY TRACKING TABLE**


Transform Reference: 4.5.8
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=65

-- There is always one or both PNDG SBMT
-- If there are both, SBMT is newer
-- At most one CMPT, CNCL can be set
-- SBMT & PNDG always happen before CMPT & CNCL
"""

from ..hl_transform import Hl_transform
from pyspark.sql.functions import col, lit, when, struct, to_date, unix_timestamp, from_unixtime, \
    explode, array_except, array, concat
from pyspark.sql import DataFrame


class Hl_DimTrade(Hl_transform):
    source_names = {'staging': ['Trade',
                                'TradeHistory'],
                    'warehouse': ['DimSecurity',
                                  'DimAccount',
                                  'StatusType',
                                  'TradeType']}

    def _get_load_df(self) -> DataFrame:
        trade_hist_pivot = self.source['TradeHistory'](1) \
            .withColumn('dte', unix_timestamp("TH_DTS")) \
            .groupBy("TH_T_ID").pivot("TH_ST_ID").max('dte') \
            .withColumn('SK_Create',
                        when(col('SBMT').isNotNull(),
                             struct(from_unixtime(col('SBMT'), 'yyyyMMdd').cast('int').alias('DateID'),
                                    from_unixtime(col('SBMT'), 'hhmmss').cast('int').alias('TimeID'))).otherwise(
                             struct(from_unixtime(col('PNDG'), 'yyyyMMdd').cast('int').alias('DateID'),
                                    from_unixtime(col('PNDG'), 'hhmmss').cast('int').alias('TimeID')))) \
            .withColumn('SK_Close',
                        when(col('CMPT').isNotNull(),
                             struct(from_unixtime(col('CMPT'), 'yyyyMMdd').cast('int').alias('DateID'),
                                    from_unixtime(col('CMPT'), 'hhmmss').cast('int').alias('TimeID'))).otherwise(
                            struct(from_unixtime(col('CNCL'), 'yyyyMMdd').cast('int').alias('DateID'),
                                   from_unixtime(col('CNCL'), 'hhmmss').cast('int').alias('TimeID'))))

        return self.source['Trade'](1) \
            .join(self.source['StatusType'](), col('T_ST_ID') == col("ST_ID"), "left") \
            .join(self.source['TradeType'](), col('T_TT_ID') == col("TT_ID"), "left") \
            .join(self.source['DimSecurity']().withColumnRenamed('EffectiveDate', 'sd_EffectiveDate')
                  .withColumnRenamed('EndDate', 'sd_EndDate'),
                  [col('T_S_SYMB') == col("Symbol"),
                   to_date('T_DTS') >= col('sd_EffectiveDate'),
                   to_date('T_DTS') <= col('sd_EndDate')], "left") \
            .join(trade_hist_pivot, col('T_ID') == col('TH_T_ID')) \
            .join(self.source['DimAccount'](), [col('T_CA_ID') == col("AccountID"),
                                              to_date('T_DTS') >= col('EffectiveDate'),
                                              to_date('T_DTS') <= col('EndDate')], 'left') \
            .select(col('T_ID').alias('TradeID'),
                    col('SK_BrokerID'),
                    col('SK_Create.DateID').alias('SK_CreateDateID'),
                    col('SK_Create.TimeID').alias('SK_CreateTimeID'),
                    col('SK_Close.DateID').alias('SK_CloseDateID'),
                    col('SK_Close.TimeID').alias('SK_CloseTimeID'),
                    col('ST_NAME').alias('Status'),
                    col('TT_NAME').alias('Type'),
                    (col('T_IS_CASH') == lit('1')).alias('CashFlag'),
                    col('SK_SecurityID'),
                    col('SK_CompanyID'),
                    col('T_QTY').alias('Quantity'),
                    col('T_BID_PRICE').alias('BidPrice'),
                    col('SK_CustomerID'),
                    col('SK_AccountID'),
                    col('T_EXEC_NAME').alias('ExecutedBy'),
                    col('T_TRADE_PRICE').alias('TradePrice'),
                    col('T_CHRG').alias('Fee'),
                    col('T_COMM').alias('Commission'),
                    col('T_TAX').alias('Tax'),
                    lit(self.BatchID).alias('BatchID'))

    def _get_alert_df(self) -> DataFrame:
        return self.load_df \
            .withColumn('trade_price_quantity', col('TradePrice') * col('Quantity')) \
            .select(explode(array_except(
                    array(when(col('trade_price_quantity') < col('Commission'),
                          struct(lit(self.CurrentTimestamp).alias('MessageDateAndTime'),
                                 lit(self.BatchID).alias('BatchID'),
                                 lit('DimTrade').alias('MessageSource'),
                                 lit('Invalid trade commission').alias('MessageText'),
                                 lit('Alert').alias('MessageType'),
                                 concat(lit('T_ID = '), col('TradeID'),
                                        lit(', C_COMM = '), col("Commission")).alias('MessageData'))),
                          when(col('trade_price_quantity') < col('Fee'),
                          struct(lit(self.CurrentTimestamp).alias('MessageDateAndTime'),
                                 lit(self.BatchID).alias('BatchID'),
                                 lit('DimTrade').alias('MessageSource'),
                                 lit('Invalid trade fee').alias('MessageText'),
                                 lit('Alert').alias('MessageType'),
                                 concat(lit('T_ID = '), col('TradeID'),
                                        lit(', T_CHRG = '), col("Fee")).alias('MessageData')))),
                    array(lit(None)))).alias('msgDetail')).select('msgDetail.*')
