"""
Historic WorkFlow Part, FactMarketHistory

TODO: Discovery on code reuse between iu_FactMarketHistory & hl_FactMarketHistory

Transform Reference: 4.5.11
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=67
"""

from ..hl_transform import Hl_transform
from pyspark.sql.functions import col, lit, when, lead, to_date, date_add, to_timestamp, \
                                  first, struct, sum, dayofyear, dayofmonth, date_format, concat
from pyspark.sql.window import Window as W
from pyspark.sql import DataFrame


class Hl_FactMarketHistory(Hl_transform):

    source_names = {'staging': ['DailyMarket'],
                    'warehouse': ['AuxFINWIRE_FIN',
                                  'DimCompany',
                                  'DimSecurity',
                                  'DimDate']}

    def _get_load_df(self) -> DataFrame:
        w_year_high = W.partitionBy(col('DM_S_SYMB')).orderBy([-col('DM_HIGH'), col('dte')]).rowsBetween(-365, 0)
        w_year_low = W.partitionBy(col('DM_S_SYMB')).orderBy([col('DM_LOW'), col('dte')]).rowsBetween(-365, 0)
        w_eps = W.partitionBy('CompanyID').orderBy(col('eps_StartDate'))

        leaper = self.source['DailyMarket'](1).sql_ctx.sparkSession.createDataFrame([(True,), (None,), ], ['leaper', ])

        company_eps = self.source['DimCompany']() \
            .join(self.source['AuxFINWIRE_FIN'](),
                  [(col('CoName') == col('Name')) | (col('CIK') == col('CompanyID')),
                   to_date(col('PTS')).between(col('EffectiveDate'), col('EndDate'))]) \
            .withColumn('eps_StartDate', to_date(col('PostingDate'), 'yyyyMMdd')) \
            .withColumn('eps_EndDate', date_add(lead(col('eps_StartDate')).over(w_eps), -1)) \
            .select(col('CompanyID').alias('eps_CompanyID'),
                    col('SK_CompanyID').alias('eps_SK_CompanyID'),
                    col('eps_StartDate'),
                    when(col('eps_EndDate').isNull(), '9999-12-31').otherwise(col('eps_EndDate')).alias('eps_EndDate'),
                    col('Year'),
                    col('Quarter'),
                    col('EPS'),
                    sum(col('EPS')).over(w_eps.rowsBetween(-3, 0)).alias('sum_eps'))

        self.CompanySymbol = self.source['DailyMarket'](self.BatchID) \
            .withColumn('dte', to_timestamp(col('DM_DATE'))) \
            .join(leaper,
                  [dayofyear(col('dte')) == lit(60),
                   dayofmonth(col('dte')) == lit(1)], 'left') \
            .withColumn('yearHigh', first(struct(col('DM_HIGH').alias('FiftyTwoWeekHigh'),
                                                 date_format(col('DM_DATE'), 'yyyyMMdd')
                                                 .alias('SK_FiftyTwoWeekHighDate'))).over(w_year_high)) \
            .withColumn('yearLow', first(struct(col('DM_LOW').alias('FiftyTwoWeekLow'),
                                                date_format(col('DM_DATE'), 'yyyyMMdd')
                                                .alias('SK_FiftyTwoWeekLowDate'))).over(w_year_low)) \
            .filter(col('leaper').isNull()) \
            .join(self.source['DimSecurity']().select(col('SK_SecurityID'),
                                                    col('SK_CompanyID'),
                                                    col('Dividend'),
                                                    col('Symbol'),
                                                    col('EffectiveDate').alias('ds_EffectiveDate'),
                                                    col('EndDate').alias('ds_EndDate')),
                  [col('DM_S_SYMB') == col('Symbol'),
                   col('DM_DATE') >= col('ds_EffectiveDate'),
                   col('DM_DATE') <= col('ds_EndDate')], 'left') \
            .join(self.source['DimDate'](), col('DM_DATE') == col('DateValue'), 'left') \
            .join(company_eps, [col('SK_CompanyID') == col('eps_SK_CompanyID'),
                                col('DM_DATE') >= col('eps_StartDate'),
                                col('DM_DATE') <= col('eps_EndDate')], 'left') \
            .select(col('SK_SecurityID'),
                    col('SK_CompanyID'),
                    date_format(col('DM_DATE'), 'yyyyMMdd').alias('SK_DateID'),
                    when(col('sum_eps').isNotNull(), col('DM_CLOSE') / col('sum_eps')).alias('PERatio'),
                    (lit(100) * col('Dividend') / col('DM_CLOSE')).alias('Yield'),
                    col('yearHigh.*'),
                    col('yearLow.*'),
                    col('DM_CLOSE').alias('ClosePrice'),
                    col('DM_HIGH').alias('DayHigh'),
                    col('DM_LOW').alias('DayLow'),
                    col('DM_VOL').alias('Volume'),
                    lit(1).alias('BatchID'),
                    col('DM_S_SYMB'))

        return self.CompanySymbol

    def _get_alert_df(self) -> DataFrame:
        """
        FactMarketHistory Alert - No earnings for company 4.5.11.3
        http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=68
        """
        return self.load_df \
                   .groupBy(col('DM_S_SYMB')) \
                   .agg(sum(col('PERatio')).alias('PERatio_sum')) \
                   .filter(col('PERatio_sum').isNull()) \
                   .select(lit(self.CurrentTimestamp).alias('MessageDateAndTime'),
                           lit(self.BatchID).alias('BatchID'),
                           lit('FactMarketHistory').alias('MessageSource'),
                           lit('No earnings for company').alias('MessageText'),
                           lit('Alert').alias('MessageType'),
                           concat(lit('DM_S_SYMB = '), col('DM_S_SYMB')).alias('MessageData'))

    def run_insert(self):
        self.set_load_df()
        if self.load_df is not None:
            self.load_df.drop('DM_S_SYMB').write.insertInto(self.target_name)
