"""
Historical Load Class is defined to satisfy the TPC-DI requirements
in 4.5 Transformation Details for the Historical Load.
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=58

Each of the sections 4.5.[1-17] will have a corresponding table that needs to
be loaded in the historic load and each will have a HistoricLoad class.

There are three dataframes that are defined within a historic load:
   load_df: The dataframe that will be inserted into the class target table
   alert_df: The dataframe of alerts (if any) that will be inserted into dimessages
   status_df: The dataframe of statuses (if any) that will be inserted into dimessages
"""

from pyspark.sql import DataFrame
from ..staging.staging import Staging
from ..warehouse.warehouse import Warehouse
from delta import DeltaTable

class Hl_transform:

    source_names: {str: [str]} = {}
    source: {str: DataFrame} = {}
    target_tbl: DataFrame = None
    audit_tbl: DataFrame = None
    load_df: DataFrame = None
    alert_df: DataFrame = None
    status_df: DataFrame = None

    def __init__(self,
                 staging: Staging,
                 warehouse: Warehouse,
                 batch_id: int,
                 batch_date: str,
                 current_timestamp: str,
                 min_effective_date: str):
        src = {}
        for stg_source_name in self.source_names['staging']:
            src[stg_source_name] = staging.Data[stg_source_name].get_df
            setattr(self, stg_source_name, staging.Data[stg_source_name].get_df)
        for wh_source_name in self.source_names['warehouse']:
            src[wh_source_name] = warehouse.Tables[wh_source_name].get_table
            setattr(self, wh_source_name, warehouse.Tables[wh_source_name].get_table)
        self.source = src
        self.BatchID = batch_id
        self.BatchDate = batch_date
        self.minEffectiveDate = min_effective_date
        self.CurrentTimestamp = current_timestamp
        self.target_db_name = warehouse.name
        self.target_tbl_name = self.__class__.__name__[3:]
        self.target_name = self.target_db_name + '.' + self.target_tbl_name
        self.target_tbl = warehouse.Tables[self.target_tbl_name].get_table
        self.dimessages_name = self.target_db_name + '.DImessages'
        self.audit_name = self.target_db_name + '.Audit'

    def _get_load_df(self) -> DataFrame:
        return getattr(self, 'load_df') if hasattr(self, 'load_df') else None

    def set_load_df(self) -> None:
        self.load_df = self._get_load_df()

    def _get_alert_df(self) -> DataFrame:
        return getattr(self, 'alert_df') if hasattr(self, 'alert_df') else None

    def set_alert_df(self) -> None:
        self.alert_df = self._get_alert_df()

    def _get_status_df(self) -> DataFrame:
        return getattr(self, 'status_df') if hasattr(self, 'status_df') else None

    def set_status_df(self) -> DataFrame:
        self.status_df = self._get_status_df()

    def run_load(self):
        self.run_insert()
        self.run_alert()
        self.run_status()

    def run_insert(self):
        self.set_load_df()
        if self.load_df is not None:
            self.load_df.write.insertInto(self.target_name)

    def run_alert(self):
        self.set_alert_df()
        if self.alert_df is not None:
            self.alert_df.write.insertInto(self.dimessages_name)

    def run_status(self):
        self.set_status_df()
        if self.status_df is not None:
            self.status_df.write.insertInto(self.dimessages_name)

    def create_views(self, merge_audit_status_only=True, short_name=True, include_batch=True):
        prefix = '' if short_name else self.target_name.split('.')[1] + '_'
        suffix = '' if include_batch else '_' + str(self.BatchID)
        self.load_df.createOrReplaceTempView(prefix + 'merge' + suffix)
        self.alert_df.createOrReplaceTempView(prefix + 'alert' + suffix)
        self.status_df.createOrReplaceTempView(prefix + 'status' + suffix)
        if ~merge_audit_status_only:
            for source_name in self.source_names['staging'].keys():
                getattr(self, source_name).createOrReplaceTempTable(source_name + suffix)
                print('View Created: ' + source_name + suffix)
            for source_name in self.source_names['warehouse'].keys():
                getattr(self, source_name).createOrReplaceTempTable(prefix + source_name + suffix)
                print('View Created: ' + source_name + suffix)
