"""
Incremental WorkFlow Part, DimCustomer
**HISTORY TRACKING TABLE**


Transform Reference: 4.6.4
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=72
"""

from ..iu_transform import Iu_transform
from pyspark.sql import DataFrame
from pyspark.sql.functions import col, lit, row_number, when, struct, upper, to_date, first, concat
from pyspark.sql.window import Window as W


class Iu_DimCustomer(Iu_transform):

    hist_track_table = True

    source_names = {'staging': ['Customer'],
                    'warehouse': ['AuxProspect',
                                  'TaxRate',
                                  'StatusType']}

    def _get_update_df(self) -> DataFrame:
        sk_customer_id_last = int(self.target_tbl().agg({"SK_CustomerID": "max"}).collect()[0][0])
        w_sk_cust = W.partitionBy(lit(1)).orderBy(lit(1))
        w_cust = W.partitionBy(col('C_ID')).orderBy(col('CDC_DSN').desc())
        w_cust_prev = W.partitionBy(col('C_ID')) \
            .orderBy(col('CDC_DSN').desc()) \
            .rowsBetween(W.currentRow, W.unboundedFollowing)

        def cust_prev(col_value, col_alias):
            return first(col_value, True).over(w_cust_prev).alias(col_alias)

        return self.source['Customer'](self.BatchID) \
            .join(self.source['StatusType'](), col('C_ST_ID') == col('ST_ID'), 'left') \
            .select(col('CDC_FLAG'),
                    row_number().over(w_cust).alias('row_num'),
                    struct(col('C_ID').alias('CustomerID'),
                           cust_prev(col('C_TAX_ID'), 'TaxID'),
                           cust_prev(col('ST_NAME'), 'Status'),
                           cust_prev(col('C_L_NAME'), 'LastName'),
                           cust_prev(col('C_F_NAME'), 'FirstName'),
                           cust_prev(col('C_M_NAME'), 'MiddleInitial'),
                           cust_prev(when(col('C_GNDR').isin(['M', 'm', 'F', 'f']),
                                          upper(col('C_GNDR'))).otherwise(lit('U')), 'Gender'),
                           cust_prev(col('C_TIER'), 'Tier'),
                           cust_prev(col('C_DOB'), 'DOB'),
                           cust_prev(col('C_ADLINE1'), 'AddressLine1'),
                           cust_prev(col('C_ADLINE2'), 'AddressLine2'),
                           cust_prev(col('C_ZIPCODE'), 'PostalCode'),
                           cust_prev(col('C_CITY'), 'City'),
                           cust_prev(col('C_STATE_PROV'), 'StateProv'),
                           cust_prev(col('C_CTRY'), 'Country'),
                           cust_prev(phoner(1), 'Phone1'),
                           cust_prev(phoner(2), 'Phone2'),
                           cust_prev(phoner(3), 'Phone3'),
                           cust_prev(col('C_EMAIL_1'), 'Email1'),
                           cust_prev(col('C_EMAIL_2'), 'Email2')).alias('CustomerDetail'),
                    cust_prev(col('C_NAT_TX_ID'), '_C_NAT_TX_ID'),
                    cust_prev(col('C_LCL_TX_ID'), '_C_LCL_TX_ID'),
                    struct(lit(True).alias('isCurrent'),
                           lit(self.BatchID).alias('BatchID'),
                           to_date(lit(self.BatchDate)).alias('EffectiveDate'),
                           lit('9999-12-31').alias('EndDate')).alias('HistTrackDetail')) \
            .filter(col('row_num') == lit(1)) \
            .join(self.source['AuxProspect'](),
                  [col('HistTrackDetail.isCurrent'),
                   upper(col('CustomerDetail.AddressLine1')) == upper(col('AddressLine1')),
                   upper(col('CustomerDetail.LastName')) == upper(col('LastName')),
                   upper(col('CustomerDetail.PostalCode')) == upper(col('PostalCode')),
                   upper(col('CustomerDetail.FirstName')) == upper(col('FirstName')),
                   upper(col('CustomerDetail.AddressLine2')) == upper(col('AddressLine2'))], 'left') \
            .join(self.source['TaxRate']().withColumnRenamed('TX_ID', 'NAT_TX_ID')
                  .withColumnRenamed('TX_NAME', 'NationalTaxRateDesc')
                  .withColumnRenamed('TX_RATE', 'NationalTaxRate'),
                  col('_C_NAT_TX_ID') == col('NAT_TX_ID'), 'left') \
            .join(self.source['TaxRate']().withColumnRenamed('TX_ID', 'LCL_TX_ID')
                  .withColumnRenamed('TX_NAME', 'LocalTaxRateDesc')
                  .withColumnRenamed('TX_RATE', 'LocalTaxRate'),
                  col('_C_LCL_TX_ID') == col('LCL_TX_ID'), 'left') \
            .select((sk_customer_id_last + row_number().over(w_sk_cust)).alias('SK_CustomerID'),
                    col("CustomerDetail.*"),
                    col("NationalTaxRateDesc"),
                    col("NationalTaxRate"),
                    col("LocalTaxRateDesc"),
                    col("LocalTaxRate"),
                    col("AgencyID"),
                    col("CreditRating"),
                    col("NetWorth"),
                    col("MarketingNameplate"),
                    col("HistTrackDetail.*"),
                    col('CDC_FLAG'))

def phoner(i):
    ctry_code = col('C_CTRY_' + str(i))
    area_code = col('C_AREA_' + str(i))
    local = col('C_LOCAL_' + str(i))
    ext = col('C_EXT_' + str(i))
    ext = when(ext.isNull(), '').otherwise(ext)
    ph1 = concat(lit('+'), ctry_code, lit(' ('), area_code, lit(') '), local, lit(' '), ext)
    ph2 = concat(lit('('), area_code, lit(') '), local, lit(' '), ext)
    ph3 = concat(local, lit(' '), ext)
    return when(ph1.isNotNull(), ph1).otherwise(
            when(ph2.isNotNull(), ph2).otherwise(when(ph3.isNotNull(), ph3)))
