"""
Incremental WorkFlow Part, DimTrade
**HISTORY TRACKING TABLE**


Transform Reference: 4.5.8
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=65

"""

from ..iu_transform import Iu_transform
from pyspark.sql.functions import explode, array_except, array, col, lit, when, struct, date_format, concat, \
    to_timestamp, row_number, last
from pyspark.sql.window import Window as W
from pyspark.sql import DataFrame
from delta import DeltaTable


class Iu_DimTrade(Iu_transform):
    source_names = {'staging': ['Trade'],
                    'warehouse': ['DimSecurity',
                                  'DimAccount',
                                  'StatusType',
                                  'TradeType']}

    def _get_update_df(self) -> DataFrame:
        w_row = W.partitionBy(col('T_ID')) \
            .orderBy(col('T_DTS').desc())
        w_trade = w_row.rowsBetween(W.currentRow, W.unboundedFollowing)

        return self.source['Trade'](self.BatchID) \
            .withColumn('T_DTS', to_timestamp(col('T_DTS'))) \
            .withColumn('T_DATE', date_format(col('T_DTS'), 'yyyyMMdd')) \
            .withColumn('T_TIME', date_format(col('T_DTS'), 'hhmmss')) \
            .withColumn('row_num', row_number().over(w_row)) \
            .withColumn('SK_CreateDateID', last(col('T_DATE')).over(w_trade)) \
            .withColumn('SK_CreateTimeID', last(col('T_TIME')).over(w_trade)) \
            .filter(col('row_num') == lit(1)) \
            .join(self.source['StatusType'](), col('T_ST_ID') == col("ST_ID"), "left") \
            .join(self.source['TradeType'](), col('T_TT_ID') == col("TT_ID"), "left") \
            .join(self.source['DimSecurity']().filter(col('isCurrent'))
                  .withColumnRenamed('EffectiveDate', 'sd_EffectiveDate')
                  .withColumnRenamed('EndDate', 'sd_EndDate'),
                  col('T_S_SYMB') == col("Symbol"), "left") \
            .join(self.source['DimAccount']().filter(col('isCurrent')),
                  col('T_CA_ID') == col("AccountID"), 'left') \
            .select(col('T_ID').alias('TradeID'),
                    col('SK_BrokerID'),
                    col('SK_CreateDateID'),
                    col('SK_CreateTimeID'),
                    col('T_DATE').alias('SK_CloseDateID'),
                    col('T_TIME').alias('SK_CloseTimeID'),
                    col('ST_NAME').alias('Status'),
                    col('TT_NAME').alias('Type'),
                    (col('T_IS_CASH') == lit('1')).alias('CashFlag'),
                    col('SK_SecurityID'),
                    col('SK_CompanyID'),
                    col('T_QTY').alias('Quantity'),
                    col('T_BID_PRICE').alias('BidPrice'),
                    col('SK_CustomerID'),
                    col('SK_AccountID'),
                    col('T_EXEC_NAME').alias('ExecutedBy'),
                    col('T_TRADE_PRICE').alias('TradePrice'),
                    col('T_CHRG').alias('Fee'),
                    col('T_COMM').alias('Commission'),
                    col('T_TAX').alias('Tax'),
                    lit(self.BatchID).alias('BatchID'))

    def _get_alert_df(self) -> DataFrame:
        return self.update_df \
            .withColumn('trade_price_quantity', col('TradePrice') * col('Quantity')) \
            .select(explode(array_except(
                    array(when(col('trade_price_quantity') < col('Commission'),
                          struct(lit(self.CurrentTimestamp).alias('MessageDateAndTime'),
                                 lit(self.BatchID).alias('BatchID'),
                                 lit('DimTrade').alias('MessageSource'),
                                 lit('Invalid trade commission').alias('MessageText'),
                                 lit('Alert').alias('MessageType'),
                                 concat(lit('T_ID = '), col('TradeID'),
                                        lit(', C_COMM = '), col("Commission")).alias('MessageData'))),
                          when(col('trade_price_quantity') < col('Fee'),
                          struct(lit(self.CurrentTimestamp).alias('MessageDateAndTime'),
                                 lit(self.BatchID).alias('BatchID'),
                                 lit('DimTrade').alias('MessageSource'),
                                 lit('Invalid trade fee').alias('MessageText'),
                                 lit('Alert').alias('MessageType'),
                                 concat(lit('T_ID = '), col('TradeID'),
                                        lit(', T_CHRG = '), col("Fee")).alias('MessageData')))),
                    array(lit(None)))).alias('msgDetail')).select('msgDetail.*')

    def run_merge(self):
        self.set_update_df()
        spark = self.update_df.sql_ctx.sparkSession
        DeltaTable.forName(spark, self.target_name).alias('tgt') \
            .merge(source=self.update_df.alias('mdf'),
                   condition=(col('tgt.`TradeID`') == col('mdf.`TradeID`'))) \
            .whenMatchedUpdate(set={"tgt.`SK_CloseDateID`": "mdf.`SK_CloseDateID`",
                                    "tgt.`SK_CloseTimeID`": "mdf.`SK_CloseTimeID`",
                                    "tgt.`Status`": "mdf.`Status`",
                                    "tgt.`TradePrice`": "mdf.`TradePrice`",
                                    "tgt.`Fee`": "mdf.`Fee`",
                                    "tgt.`Commission`": "mdf.`Commission`",
                                    "tgt.`Tax`": "mdf.`Tax`"}) \
            .whenNotMatchedInsertAll().execute()
