"""
Incremental Update Class is defined to satisfy the TPC-DI requirements
in 4.6 Transformation Details for Incremental Updates
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=71

Each of the sections 4.6.[1-17] will have a corresponding table that needs to
be updated in the historic load and each will have a HistoricLoad class.

There are three dataframes that are defined within a historic load:
   update_df: The dataframe that will be updated into the class target table
   alert_df: The dataframe of alerts (if any) that will be inserted into dimessages
   status_df: The dataframe of statuses (if any) that will be inserted into dimessages
"""

from pyspark.sql import DataFrame
from ..staging.staging import Staging
from ..warehouse.warehouse import Warehouse
from pyspark.sql.functions import col, lit, when, struct, explode, array, date_add
from delta import DeltaTable


class Iu_transform:
    hist_track_table = False

    source_names: {str: [str]} = {}
    source: {str: DataFrame} = {}
    audit_tbl: DataFrame = None
    update_df: DataFrame = None
    alert_df: DataFrame = None
    status_df: DataFrame = None

    def __init__(self,
                 staging: Staging,
                 warehouse: Warehouse,
                 batch_id: int,
                 batch_date: str,
                 current_timestamp: str,
                 min_effective_date: str):
        src = {}
        for stg_source_name in self.source_names['staging']:
            src[stg_source_name] = staging.Data[stg_source_name].get_df
            setattr(self, stg_source_name, staging.Data[stg_source_name].get_df)
        for wh_source_name in self.source_names['warehouse']:
            src[wh_source_name] = warehouse.Tables[wh_source_name].get_table
            setattr(self, wh_source_name, warehouse.Tables[wh_source_name].get_table)
        self.source = src
        self.BatchID = batch_id
        self.BatchDate = batch_date
        self.minEffectiveDate = min_effective_date
        self.CurrentTimestamp = current_timestamp
        self.target_db_name = warehouse.name
        self.target_tbl_name = self.__class__.__name__[3:]
        self.target_name = self.target_db_name + '.' + self.target_tbl_name
        self.target_tbl = warehouse.Tables[self.target_tbl_name].get_table
        self.dimessages_name = self.target_db_name + '.DImessages'
        self.audit_name = self.target_db_name + '.Audit'

    def _get_update_df(self) -> DataFrame:
        return getattr(self, 'update_df') if hasattr(self, 'update_df') else None

    def set_update_df(self) -> None:
        self.update_df = self._get_update_df()

    def _get_alert_df(self) -> DataFrame:
        return getattr(self, 'alert_df') if hasattr(self, 'alert_df') else None

    def set_alert_df(self) -> None:
        self.alert_df = self._get_alert_df()

    def _get_status_df(self) -> DataFrame:
        return getattr(self, 'status_df') if hasattr(self, 'status_df') else None

    def set_status_df(self) -> None:
        self.status_df = self._get_status_df()

    def run_update(self):
        self.run_merge()
        self.run_alert()
        self.run_status()

    def run_merge(self):
        self.set_update_df()
        spark = self.update_df.sql_ctx.sparkSession
        if self.hist_track_table:
            insert_cols = self.update_df.columns[:-1]
            target_id_col = self.__class__.__name__.split('Dim')[1] + 'ID'
            sk_target_id_col = 'SK_' + target_id_col
            update_cols_not_null = (target_id_col, 'EffectiveDate')
            update_cols = [col(c) if c in update_cols_not_null else lit(None).alias(c) for i, c in
                           enumerate(insert_cols)]
            mdf = self.update_df.select(when(col('CDC_FLAG') == lit('U'),
                                             array(struct(insert_cols),
                                                   struct(update_cols))).otherwise(
                array(struct(insert_cols))).alias('arr')) \
                .select(explode(col('arr')).alias('rec')).select(col('rec.*'))
            condition = ((col(f'tgt.`{target_id_col}`') == col(f'mdf.`{target_id_col}`')) &
                         col('tgt.`isCurrent`') & col(f'mdf.`{sk_target_id_col}`').isNull())
            DeltaTable.forName(spark, self.target_name).alias('tgt') \
                .merge(source=mdf.alias('mdf'),
                       condition=condition) \
                .whenMatchedUpdate(set={"tgt.`isCurrent`": lit(False),
                                        "tgt.`EndDate`": date_add(col('mdf.`EffectiveDate`'), -1)}) \
                .whenNotMatchedInsertAll().execute()
        else:
            self.update_df.write.insertInto(self.target_name)

    def run_alert(self):
        self.set_alert_df()
        if self.alert_df is not None:
            self.alert_df.write.insertInto(self.dimessages_name)

    def run_status(self):
        self.set_status_df()
        if self.status_df is not None:
            self.status_df.write.insertInto(self.dimessages_name)

    def create_views(self, merge_audit_status_only=True, short_name=True, include_batch=True):
        prefix = '' if short_name else self.target_name.split('.')[1] + '_'
        suffix = '' if include_batch else '_' + str(self.BatchID)
        self.update_df.createOrReplaceTempView(prefix + 'update' + suffix)
        self.alert_df.createOrReplaceTempView(prefix + 'alert' + suffix)
        self.status_df.createOrReplaceTempView(prefix + 'status' + suffix)
        if ~merge_audit_status_only:
            for source_name in self.source_names['staging'].keys():
                getattr(self, source_name).createOrReplaceTempTable(source_name + suffix)
                print('View Created: ' + source_name + suffix)
            for source_name in self.source_names['warehouse'].keys():
                getattr(self, source_name).createOrReplaceTempTable(prefix + source_name + suffix)
                print('View Created: ' + source_name + suffix)
