"""
Historical Load Class is defined to satisfy the TPC-DI requirements
in 4.5 Transformation Details for the Historical Load.
http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=14

Each of the sections 4.5.[1-17] will have a corresponding table that needs to
be updated in the historic load and each will have a HistoricLoad class.

This class has some conventions to categorize dataframe:
  sdf: These are dataframes are the source dataframes for this HistoricLoad
  tdf: These are temporary dataframes defined for a logic point to be able
       to inspect a table or defined to be used as contributing dataframe
       to an idf or mdf.
  idf: This is an intermediate dataframe - this is different from a tdf in
       the fact that it intended to be extracted and used as the source in
       down stream HistoricLoads
  mdf: This is a merge(insert) dataframe, it has all fields in expected format
       for a merge or insert operation
  sdf: This is an alert dataframe, it has the definition of alert messages to
       write into DImessages
"""

import pyspark
from ..staging.staging import Staging
from ..warehouse.warehouse import Warehouse
from delta import DeltaTable

class Transform:

    source_names: {str: [str]}
    source: {str: pyspark.sql.DataFrame}
    target: pyspark.sql.DataFrame
    audit: pyspark.sql.DataFrame
    merge = None
    alert = None
    status = None

    def __init__(self,
                 staging: Staging,
                 warehouse: Warehouse,
                 batch_id: int,
                 batch_date: str,
                 current_timestamp: str,
                 min_effective_date: str):
        src = {}
        for stg_source_name in self.source_names['staging']:
            src[stg_source_name] = staging.Data[stg_source_name].get_df(batch_id)
            setattr(self, stg_source_name, staging.Data[stg_source_name].get_df(batch_id))
        for wh_source_name in self.source_names['warehouse']:
            src[wh_source_name] = warehouse.Tables[wh_source_name].get_table()
            setattr(self, wh_source_name, warehouse.Tables[wh_source_name].get_table())
        self.source = src
        self.BatchID = batch_id
        self.BatchDate = batch_date
        self.minEffectiveDate = min_effective_date
        self.CurrentTimestamp = current_timestamp
        self.target_name = warehouse.name + '.' + self.__class__.__name__[3:]
        self.target = warehouse.Tables[self.__class__.__name__[3:]].get_table()
        self.dimessages_name = warehouse.name + '.DImessages'
        # self.merge = self._get_merge()

    def _get_merge(self) -> pyspark.sql.DataFrame:
        return getattr(self, 'target') if hasattr(self, 'target') else None

    def _get_alert(self) -> pyspark.sql.DataFrame:
        return getattr(self, 'audit') if hasattr(self, 'audit') else None

    def _get_status(self) -> pyspark.sql.DataFrame:
        return getattr(self, 'status') if hasattr(self, 'status') else None

    def run_load(self):
        self.run_insert()
        self.run_update()
        self.run_alert()
        self.run_status()

    def run_update(self):
        pass

    def run_insert(self):
        print(self.target_name)
        self.merge = self._get_merge()
        self.merge.write.insertInto(self.target_name)

    def run_alert(self):
        self.alert = self._get_alert()
        if self.alert is not None:
            self.alert.write.insertInto(self.dimessages_name)

    def run_status(self):
        self.status = self._get_status()
        if self.status is not None:
            self.status.write.insertInto(self.dimessages_name)

    def create_views(self, short_name=True, merge_audit_status_only=True):
        prefix = '' if short_name else self.target_name.split('.')[1] + '_'
        self.merge.createOrReplaceTempView(prefix + 'merge')
        self.alert.createOrReplaceTempView(prefix + 'alert')
        self.status.createOrReplaceTempView(prefix + 'status')
        if ~merge_audit_status_only:
            for source_name in self.source_names['staging'].keys() + self.source_names['warehouse'].keys():
                getattr(self, source_name).createOrReplaceTempTable(source_name)

    def run_purge(self):
        print(self.target_name)
        spark = self.target.sql_ctx.sparkSession
        spark.sql(f'DELETE FROM {self.target_name} WHERE 1=1')
        DeltaTable.forName(spark, self.target_name).vacuum()
