"""
Batch Validation Query

There are

Batch Validation Query

TODO: This query takes a long time due to the metastore lookup. Opportunity to run
get_table in threads to improve runtime.


Req: 7.4 Batch Validation Query
Ref: http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=38
"""

from pyspark.sql.functions import col, lit, count, current_timestamp, max
from pyspark.sql import SparkSession
from functools import reduce
from pyspark.sql import DataFrame


class Validation:
    validation: DataFrame

    basic_row_count_tbl_names = (
        'DimAccount', 'DimBroker', 'DimCompany', 'DimCustomer', 'DimDate', 'DimSecurity', 'DimTime', 'DimTrade',
        'FactCashBalances', 'FactHoldings', 'FactMarketHistory', 'FactWatches',
        'FactWatches', 'Financial', 'Industry', 'Prospect', 'StatusType', 'TaxRate', 'TradeType')

    joined_row_count_tbl_joins = \
        (('FactCashBalances', ('Account', 'Customer', 'Broker', 'Date')),
         ('FactHoldings', ('Account', 'Customer', 'Broker', 'Date', 'Time', 'Company', 'Security')),
         ('FactMarketHistory', ('Date', 'Company', 'Security')),
         ('FactWatches', ('Customer', 'Security')))

    def __init__(self, warehouse):
        self.spark = SparkSession.builder.getOrCreate()
        self.wh = warehouse

    def _basic_row_count(self, tbl_name):
        return self.wh.Tables[tbl_name].get_table().agg(count(lit(1)).alias('MessageData')) \
            .select(lit(tbl_name).alias('MessageSource'),
                    lit('Row count').alias('MessageText'),
                    col('MessageData'))

    def _get_basic_row_counts(self):
        return [self._basic_row_count(tn) for tn in self.basic_row_count_tbl_names]

    def _joined_row_count(self, tbl_joins):
        tbl_name = tbl_joins[0]
        joins = tbl_joins[1]
        joined_df = self.wh.Tables[tbl_name].get_table()
        for j in joins:
            joined_df = joined_df.join(self.wh.Tables['Dim' + j].get_table(), 'SK_' + j + 'ID')
        return joined_df.agg(count(lit(1)).alias('MessageData')) \
            .select(lit(tbl_name).alias('MessageSource'),
                    lit('Row count joined').alias('MessageText'),
                    col('MessageData'))

    def _get_joined_row_counts(self):
        return [self._joined_row_count(tj) for tj in self.joined_row_count_tbl_joins]

    def _get_additional_info_counts(self):
        additional_info_dimcustomer = self.wh.Tables['DimCustomer'].get_table() \
            .filter(col('isCurrent') & (col('Status') == lit('Inactive'))) \
            .agg(count(lit(1)).alias('MessageData')) \
            .select(lit('DimCustomer').alias('MessageSource'),
                    lit('Inactive customers').alias('MessageText'),
                    col('MessageData'))
        additional_info_factwatches = self.wh.Tables['FactWatches'].get_table() \
            .filter(col('SK_DateID_DateRemoved').isNotNull()) \
            .agg(count(lit(1)).alias('MessageData')) \
            .select(lit('FactWatches').alias('MessageSource'),
                    lit('Inactive watches').alias('MessageText'),
                    col('MessageData'))
        return [additional_info_dimcustomer, additional_info_factwatches]

    def _set_validation_df(self):
        batch_id = self.wh.Tables['DImessages'].get_table().agg(max(col('BatchID')).alias('BatchID')).collect()[0][0]
        self.validation_df = reduce(DataFrame.unionAll, self._get_basic_row_counts() +
                                    self._get_joined_row_counts() +
                                    self._get_additional_info_counts()) \
            .select(current_timestamp().alias('MessageDateAndTime'),
                    lit(batch_id).alias('BatchID'),
                    col('MessageSource'),
                    col('MessageText'),
                    lit('Validation').alias('MessageType'),
                    col('MessageData'))

    def run_validation(self):
        self._set_validation_df()
        self.validation_df.write.insertInto(self.wh.Tables['DImessages'].name)
