"""
Provide Graphic to list tables in Warehouse

All of the tables in the Data Warehouse are listed 1.4 Destination Data Model. However, presentation and 
ease of inspection would be tedious for users either not familiar with Databricks or the TPC-DI benchmark.
Thus, we'll provide a graphic that is visually similar to Figure 1.4-1 and will hyperlink to the data warehouse 
tables that would be created with the given sf.

NOTE: The code segment, gvDependency.py, is not used in any phases of operation tasks.
It is provided only for discovery in interactive notebook sessions.

TPC-DI Requirement: 1.4
TPC-DI Ref: http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=17
"""


def get_wh_html(wh_name):
    import re
    from graphviz import Digraph

    dot = Digraph('tpcdi')
    dot.graph_attr['rankdir'] = 'LR'
    dot.attr('node', shape='rectangle')

    def tbl_link(tbl_name, ttip=''):
        return {'tooltip': ttip, 'href': f'#table/hive_metastore/{wh_name}/{tbl_name}', 'target': "_blank"}

    with dot.subgraph(name='cluster_warehouse') as c:
        c.body.append('label="Warehouse Database"')
        with c.subgraph(name='cluster_warehouse_aux') as a:
            a.body.append('label="Auxiliary Tables"')
            a.body.append('style="bold"')
            a.body.append('color="#FCFF99"')
            a.node('AuxFINWIRE', 'AuxFINWIRE', fillcolor='#FCFF99', style='filled', **tbl_link('auxfinwire', '3.4.8'))
            a.node('AuxCustomerMgmt', 'AuxCustomerMgmt', fillcolor='#FCFF99', style='filled',
                   **tbl_link('auxcustomermgmt', '3.4.8'))
            a.node('AuxProspect', 'AuxProspect', fillcolor='#FCFF99', style='filled',
                   **tbl_link('auxprospect', '3.4.8'))
        with c.subgraph(name='cluster_warehouse_fact') as f:
            f.body.append('label="Fact Tables"')
            f.body.append('style="bold"')
            f.body.append('color="#CAD9EF"')
            f.node('FactCashBalances', 'FactCashBalances', fillcolor='#CAD9EF', style='filled',
                   **tbl_link('factcashbalances', '3.2.9'))
            f.node('FactHoldings', 'FactHoldings', fillcolor='#CAD9EF', style='filled',
                   **tbl_link('factholdings', '3.2.10'))
            f.node('FactWatches', 'FactWatches', fillcolor='#CAD9EF', style='filled',
                   **tbl_link('factwatches', '3.2.12'))
            f.node('FactMarketHistory', 'FactMarketHistory', fillcolor='#CAD9EF', style='filled',
                   **tbl_link('factmarkethistory', '3.2.11'))
            f.node('Prospect', 'Prospect', fillcolor='#CAD9EF', style='filled', **tbl_link('Prospect', '3.2.15'))
        c.node('DimTrade', 'DimTrade', fillcolor='#CAD9EF:#FAEADC', style='filled', gradientangle='0',
               **tbl_link('dimtrade', '3.2.7.1'))
        with c.subgraph(name='cluster_warehouse_dim') as d:
            d.body.append('label="Dimension Tables"')
            d.body.append('style="bold"')
            d.body.append('color="#FAEADC"')
            d.node('DimCustomer', 'DimCustomer', fillcolor='#FAEADC', style='filled',
                   **tbl_link('dimcustomer', '3.2.4'))
            d.node('DimAccount', 'DimAccount', fillcolor='#FAEADC', style='filled', **tbl_link('dimaccount', '3.2.1'))
            d.node('DimBroker', 'DimBroker', fillcolor='#FAEADC', style='filled', **tbl_link('dimbroker', '3.2.2'))
            d.node('DimSecurity', 'DimSecurity', fillcolor='#FAEADC', style='filled',
                   **tbl_link('dimsecurity', '3.2.6'))
            d.node('DimCompany', 'DimCompany', fillcolor='#FAEADC', style='filled', **tbl_link('dimcompany', '3.2.3'))
        c.node('DimDate', 'DimDate', fillcolor='#FAEADC:#ECF1E0', style='filled', gradientangle='0',
               **tbl_link('dimdate', '3.2.5'))
        c.node('DimTime', 'DimTime', fillcolor='#FAEADC:#ECF1E0', style='filled', gradientangle='0',
               **tbl_link('dimtime', '3.2.7'))
        with c.subgraph(name='cluster_warehouse_ref') as r:
            r.body.append('label="Reference Tables"')
            r.body.append('style="bold"')
            r.body.append('color="#ECF1E0"')
            r.node('TradeType', 'TradeType', fillcolor='#ECF1E0', style='filled', **tbl_link('tradetype', '3.2.18'))
            r.node('StatusType', 'StatusType', fillcolor='#ECF1E0', style='filled', **tbl_link('statustype', '3.2.16'))
            r.node('TaxRate', 'TaxRate', fillcolor='#ECF1E0', style='filled', **tbl_link('taxrate', '3.2.17'))
            r.node('Industry', 'Industry', fillcolor='#ECF1E0', style='filled', **tbl_link('industry', '3.2.13'))
            r.node('Financial', 'Financial', fillcolor='#ECF1E0', style='filled', **tbl_link('financial', '3.2.14'))
        with c.subgraph(name='cluster_warehouse_di') as o:
            o.body.append('label="DI Operational Tables"')
            o.body.append('style="bold"')
            o.body.append('color="#E5E0EB"')
            o.node('DImessages', 'DImessages', fillcolor='#E5E0EB', style='filled', **tbl_link('dimessages', '3.2.8'))
            o.node('Audit', 'Audit', fillcolor='#E5E0EB', style='filled', **tbl_link('audit', '3.2.19'))

        dot.edge('AuxFINWIRE', 'FactCashBalances', style="invis")
        dot.edge('AuxFINWIRE', 'FactHoldings', style="invis")
        dot.edge('AuxFINWIRE', 'FactWatches', style="invis")
        dot.edge('AuxFINWIRE', 'FactMarketHistory', style="invis")
        dot.edge('AuxFINWIRE', 'Prospect', style="invis")

        dot.edge('AuxCustomerMgmt', 'FactCashBalances', style="invis")
        dot.edge('AuxCustomerMgmt', 'FactHoldings', style="invis")
        dot.edge('AuxCustomerMgmt', 'FactWatches', style="invis")
        dot.edge('AuxCustomerMgmt', 'FactMarketHistory', style="invis")
        dot.edge('AuxCustomerMgmt', 'Prospect', style="invis")

        dot.edge('AuxProspect', 'FactCashBalances', style="invis")
        dot.edge('AuxProspect', 'FactHoldings', style="invis")
        dot.edge('AuxProspect', 'FactWatches', style="invis")
        dot.edge('AuxProspect', 'FactMarketHistory', style="invis")
        dot.edge('AuxProspect', 'Prospect', style="invis")

        dot.edge('FactCashBalances', 'DimTrade', style="invis")
        dot.edge('FactHoldings', 'DimTrade', style="invis")
        dot.edge('FactWatches', 'DimTrade', style="invis")
        dot.edge('FactMarketHistory', 'DimTrade', style="invis")
        dot.edge('Prospect', 'DimTrade', style="invis")

        dot.edge('DimTrade', 'DimCustomer', style="invis")
        dot.edge('DimTrade', 'DimAccount', style="invis")
        dot.edge('DimTrade', 'DimBroker', style="invis")
        dot.edge('DimTrade', 'DimSecurity', style="invis")
        dot.edge('DimTrade', 'DimCompany', style="invis")

        dot.edge('DimCustomer', 'DimDate', style="invis")
        dot.edge('DimAccount', 'DimDate', style="invis")
        dot.edge('DimBroker', 'DimDate', style="invis")
        dot.edge('DimSecurity', 'DimDate', style="invis")
        dot.edge('DimCompany', 'DimDate', style="invis")

        dot.edge('DimCustomer', 'DimTime', style="invis")
        dot.edge('DimAccount', 'DimTime', style="invis")
        dot.edge('DimBroker', 'DimTime', style="invis")
        dot.edge('DimSecurity', 'DimTime', style="invis")
        dot.edge('DimCompany', 'DimTime', style="invis")

        dot.edge('DimDate', 'TradeType', style="invis")
        dot.edge('DimDate', 'StatusType', style="invis")
        dot.edge('DimDate', 'TaxRate', style="invis")
        dot.edge('DimDate', 'Industry', style="invis")
        dot.edge('DimDate', 'Financial', style="invis")

        dot.edge('DimTime', 'TradeType', style="invis")
        dot.edge('DimTime', 'StatusType', style="invis")
        dot.edge('DimTime', 'TaxRate', style="invis")
        dot.edge('DimTime', 'Industry', style="invis")
        dot.edge('DimTime', 'Financial', style="invis")

        dot.edge('TradeType', 'DImessages', style="invis")
        dot.edge('StatusType', 'DImessages', style="invis")
        dot.edge('TaxRate', 'DImessages', style="invis")
        dot.edge('Industry', 'DImessages', style="invis")
        dot.edge('Financial', 'DImessages', style="invis")

        dot.edge('TradeType', 'Audit', style="invis")
        dot.edge('StatusType', 'Audit', style="invis")
        dot.edge('TaxRate', 'Audit', style="invis")
        dot.edge('Industry', 'Audit', style="invis")
        dot.edge('Financial', 'Audit', style="invis")

    html = dot._repr_image_svg_xml()
    html = re.sub(r'<svg width=\"\d*pt\" height=\"\d*pt\"',
                  '<div style="text-align:center;"><svg width="900pt" aligned=center', html)
    html = re.sub(r'font-size=\"14.00\">Fact Tables</text>', 'font-size=\"12.00\">Fact Tables</text>', html)
    html = re.sub(r'font-size=\"14.00\">Dimension Tables</text>', 'font-size=\"12.00\">Dimension Tables</text>', html)
    html = re.sub(r'font-size=\"14.00\">Reference Tables</text>', 'font-size=\"12.00\">Reference Tables</text>', html)
    html = re.sub(r'font-size=\"14.00\">DI Operational Tables</text>',
                  'font-size=\"12.00\">DI Operational Tables</text>', html)
    html = re.sub(r'stroke-width=\"2\"', 'stroke-width=\"4\"', html)
    html = re.sub(r'</svg>', '</div</svg>', html)

    return html
