"""
AuxFINWIRE - An auxiliary table for FINWIRE

While the inclusion of an auxiliary table for a multi-schema source isn't required, it is
the most likely approach that would be used in an actual production environment. Thus,
we'll use that approach here as well.

First we will create a source table that has all of the original source files in original
schema. However, we will then also add view to facilitate inspection of the individual
schema types. This will improve readability and mitigate technical debt.

TPC-DI Requirements: 2.2.2.8, 3.4.8
TPC-DI Ref: http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=29
TPC-DI Ref: http://tpc.org/tpc_documents_current_versions/pdf/tpc-di_v1.1.0.pdf#page=48
"""

from ..whTable import WhTable
from pyspark.sql.types import *
from delta import DeltaTable


# noinspection SqlResolve
class AuxFINWIRE(WhTable):
    definition = [("PTS", TimestampType(), False, "Posting Timestamp"),
                  ("RecType", StringType(), False, "Record Type; CMP, SEC, FIN"),
                  ("CMP", StructType([StructField('CompanyName', StringType()),
                                      StructField('CIK', StringType()),
                                      StructField('Status', StringType()),
                                      StructField('IndustryID', StringType()),
                                      StructField('SPrating', StringType()),
                                      StructField('FoundingDate', DateType()),
                                      StructField('AddrLine1', StringType()),
                                      StructField('AddrLine2', StringType()),
                                      StructField('PostalCode', StringType()),
                                      StructField('City', StringType()),
                                      StructField('StateProvince', StringType()),
                                      StructField('Country', StringType()),
                                      StructField('CEOname', StringType()),
                                      StructField('Description', StringType())]), True, "CMP Record"),
                  ("SEC", StructType([StructField('Symbol', StringType()),
                                      StructField('IssueType', StringType()),
                                      StructField('Status', StringType()),
                                      StructField('Name', StringType()),
                                      StructField('ExID', StringType()),
                                      StructField('ShOut', IntegerType()),
                                      StructField('FirstTradeDate', DateType()),
                                      StructField('FirstTradeExchg', DateType()),
                                      StructField('Dividend', DecimalType(10, 2)),
                                      StructField('CIK', DecimalType(11, 0)),
                                      StructField('CoName', StringType())]), True, "SEC Record"),
                  ("FIN", StructType([StructField('Year', ShortType()),
                                      StructField('Quarter', ByteType()),
                                      StructField('QtrStartDate', DateType()),
                                      StructField('PostingDate', DateType()),
                                      StructField('Revenue', DecimalType(14, 2)),
                                      StructField('Earnings', DecimalType(14, 2)),
                                      StructField('EPS', DecimalType(9, 2)),
                                      StructField('DilutedEPS', DecimalType(9, 2)),
                                      StructField('Margin', DecimalType(4, 2)),
                                      StructField('Inventory', DecimalType(14, 2)),
                                      StructField('Assets', DecimalType(14, 2)),
                                      StructField('Liabilities', DecimalType(14, 2)),
                                      StructField('ShOut', IntegerType()),
                                      StructField('DilutedShOut', IntegerType()),
                                      StructField('CIK', DecimalType(11, 0)),
                                      StructField('CoName', StringType())]), True, "FIN Record")]

    def create_table(self) -> None:
        dt = DeltaTable.createIfNotExists(self.spark).tableName(self.name)
        for c in self.definition:
            dt.addColumn(*c[:-1], comment=c[-1])
        dt.partitionedBy('RecType')
        dt.execute()
        for rt in ('CMP', 'SEC', 'FIN'):
            sql_create_view = f'CREATE OR REPLACE VIEW {self.name}_{rt} AS '
            sql_create_view += f'SELECT PTS, {rt}.* FROM {self.name} WHERE RecType = "{rt}" '
            self.spark.sql(sql_create_view)
